ServerFeatures

Struct ServerFeatures 

pub struct ServerFeatures(/* private fields */);
Features supported by an NBD server implementation.

Derived from transmission flags but excluding administrative flags like HAS_FLAGS and READ_ONLY. These flags allow the server to advertise which features it supports.

For each flag, the server:

  • MAY set the flag for features it supports
  • MUST NOT set the flag for features it does not support
  • The client MUST NOT use a feature documented as ‘exposed’ by a flag unless that flag was set

Available flags:

  • ServerFeatures::SEND_FLUSH (0x0004): Exposes support for NBD_CMD_FLUSH.
  • ServerFeatures::SEND_FUA (0x0008): Exposes support for NBD_CMD_FLAG_FUA (Force Unit Access).
  • ServerFeatures::ROTATIONAL (0x0010): Indicates the export has characteristics of a rotational medium. The client MAY schedule I/O accesses accordingly.
  • ServerFeatures::SEND_TRIM (0x0020): Exposes support for NBD_CMD_TRIM.
  • ServerFeatures::SEND_WRITE_ZEROES (0x0040): Exposes support for NBD_CMD_WRITE_ZEROES and NBD_CMD_FLAG_NO_HOLE.
  • ServerFeatures::SEND_DF (0x0080): Do not fragment a structured reply. Indicates the server supports the NBD_CMD_FLAG_DF request flag.
  • ServerFeatures::CAN_MULTI_CONN (0x0100): Indicates that the server operates without cache or with a shared cache, making FLUSH and FUA operations visible across all connections. Without this flag, clients SHOULD NOT multiplex commands over multiple connections.
  • ServerFeatures::SEND_RESIZE (0x0200): Exposes support for the experimental RESIZE extension.
  • ServerFeatures::SEND_CACHE (0x0400): Documents that the server understands NBD_CMD_CACHE. Note that some servers may support the command without this bit, and this flag doesn’t guarantee the command will succeed.
  • ServerFeatures::SEND_FAST_ZERO (0x0800): Allows clients to detect if NBD_CMD_WRITE_ZEROES is faster than a corresponding write via the NBD_CMD_FLAG_FAST_ZERO request flag.
  • ServerFeatures::BLOCK_STATUS_PAYLOAD (0x1000): Defined by the experimental EXTENDED_HEADERS extension.

Implementations

Source

impl ServerFeatures

pub const SEND_FLUSH: Self

pub const SEND_FUA: Self

pub const ROTATIONAL: Self

pub const SEND_TRIM: Self

pub const SEND_WRITE_ZEROES: Self

pub const SEND_DF: Self

pub const CAN_MULTI_CONN: Self

pub const SEND_RESIZE: Self

pub const SEND_CACHE: Self

pub const SEND_FAST_ZERO: Self

pub const BLOCK_STATUS_PAYLOAD: Self

Source

impl ServerFeatures

pub const fn empty() -> Self

Get a flags value with all bits unset.

pub const fn all() -> Self

Get a flags value with all known bits set.

pub const fn bits(&self) -> u16

Get the underlying bits value.

The returned value is exactly the bits set in this flags value.

pub const fn from_bits(bits: u16) -> Option<Self>

Convert from a bits value.

This method will return None if any unknown bits are set.

pub const fn from_bits_truncate(bits: u16) -> Self

Convert from a bits value, unsetting any unknown bits.

pub const fn from_bits_retain(bits: u16) -> Self

Convert from a bits value exactly.

pub fn from_name(name: &str) -> Option<Self>

Get a flags value with the bits of a flag with the given name set.

This method will return None if name is empty or doesn’t correspond to any named flag.

pub const fn is_empty(&self) -> bool

Whether all bits in this flags value are unset.

pub const fn is_all(&self) -> bool

Whether all known bits in this flags value are set.

pub const fn intersects(&self, other: Self) -> bool

Whether any set bits in a source flags value are also set in a target flags value.

pub const fn contains(&self, other: Self) -> bool

Whether all set bits in a source flags value are also set in a target flags value.

pub fn insert(&mut self, other: Self)

The bitwise or (|) of the bits in two flags values.

pub fn remove(&mut self, other: Self)

The intersection of a source flags value with the complement of a target flags value (&!).

This method is not equivalent to self & !other when other has unknown bits set. remove won’t truncate other, but the ! operator will.

pub fn toggle(&mut self, other: Self)

The bitwise exclusive-or (^) of the bits in two flags values.

pub fn set(&mut self, other: Self, value: bool)

Call insert when value is true or remove when value is false.

pub const fn intersection(self, other: Self) -> Self

The bitwise and (&) of the bits in two flags values.

pub const fn union(self, other: Self) -> Self

The bitwise or (|) of the bits in two flags values.

pub const fn difference(self, other: Self) -> Self

The intersection of a source flags value with the complement of a target flags value (&!).

This method is not equivalent to self & !other when other has unknown bits set. difference won’t truncate other, but the ! operator will.

pub const fn symmetric_difference(self, other: Self) -> Self

The bitwise exclusive-or (^) of the bits in two flags values.

pub const fn complement(self) -> Self

The bitwise negation (!) of the bits in a flags value, truncating the result.

Source

impl ServerFeatures

pub const fn iter(&self) -> Iter<ServerFeatures>

Yield a set of contained flags values.

Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.

pub const fn iter_names(&self) -> IterNames<ServerFeatures>

Yield a set of contained named flags values.

This method is like iter, except only yields bits in contained named flags. Any unknown bits, or bits not corresponding to a contained flag will not be yielded.